#
#   MailScanner - SMTP E-Mail Virus Scanner
#   Copyright (C) 2002  Julian Field
#
#   $Id: Log.pm,v 1.14 2002/11/10 15:02:15 jkf Exp $
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   The author, Julian Field, can be contacted by email at
#      Jules@JulianField.net
#   or by paper mail at
#      Julian Field
#      Dept of Electronics & Computer Science
#      University of Southampton
#      Southampton
#      SO17 1BJ
#      United Kingdom
#

###########################################################
# Syslog library calls
###########################################################

package MailScanner::Log;

use strict;
use Sys::Syslog;
use Carp;
use vars qw($LogType $Banner);

$LogType |= 'syslog';

sub Configure {
  my($banner,$type) = @_;

  $Banner = $banner?$banner:undef;
  $LogType = $type?$type:'syslog';
}

sub Start {
  my($name, $facility) = @_;

  if ($LogType eq 'syslog') {
    # Do this in an eval so it can fail quietly if setlogsock
    # is not supported in the installed version of Sys::Syslog
    #eval { $SIG{'__DIE__'} = 'IGNORE';
    #       Sys::Syslog::setlogsock('unix');
    #     }; # Doesn't need syslogd -r
    #$SIG{'__DIE__'} = 'DEFAULT';
    eval { Sys::Syslog::setlogsock('unix'); }; # This may fail!
    Sys::Syslog::openlog($name, 'pid, nowait', $facility);
  }
  
  if (defined $Banner) {
    InfoLog($Banner);
  }
}

sub Stop {
    Sys::Syslog::closelog() if $LogType eq 'syslog';
}

sub DieLog {
  # closelog changes $! in @_
  my(@x) = @_;

  my $logmessage = sprintf shift @x, @x;

  foreach(split /\n/,$logmessage) {
    Sys::Syslog::syslog('err', $_) if $LogType eq 'syslog';
  }

  Sys::Syslog::closelog() if $LogType eq 'syslog';

  croak "$logmessage";
}

sub WarnLog {
  my(@x) = @_;
  my $logmessage = sprintf shift @x, @x;

  foreach(split /\n/,$logmessage) {
    Sys::Syslog::syslog('warning', @_) if $LogType eq 'syslog';
  }

  carp $logmessage if $LogType eq 'stderr';
}

sub InfoLog {
  my(@x) = @_;
  my $logmessage = sprintf shift @x, @x;

  foreach(split /\n/,$logmessage) {
    Sys::Syslog::syslog('info', @_) if $LogType eq 'syslog';
  }

  print STDERR "@x\n" if $LogType eq 'stderr';
}

sub DebugLog {
  my(@x) = @_;
  if (MailScanner::Config::Value('debug')) {
    my $logmessage = sprintf shift @x, @x;

    foreach(split /\n/,$logmessage) {
      Sys::Syslog::syslog('debug', @_) if $LogType eq 'syslog';
    }

    print STDERR "@x\n" if $LogType eq 'stderr';
  }
}

1;
